from fastmcp import FastMCP, Context
from ..utils.directory_manager import manager
from ..utils.formatters import format_mcp_list


def register_queue_tools(mcp: FastMCP):

    @mcp.tool()
    async def list_queues(
        ctx: Context,
        fields: list[str] | None = None,
        per_page: int | None = None,
        use_cache: bool = True,
    ) -> str:
        """List all available queues.

        Args:
            fields: Optional list of additional fields (e.g. ["description", "issueTypesConfig"])
            per_page: Results per page (if not using cache)
            use_cache: Whether to use local cache (default: True). Note: per_page is ignored if using cache.
        """
        tracker = ctx.lifespan_context["tracker"]

        # Cache only the full list (without per_page filter)
        if per_page is None:
            queues = await manager.get("queues", tracker.queues.list, force=not use_cache)
        else:
            queues = await tracker.queues.list(per_page=per_page)

        return format_mcp_list(
            queues, "Queues",
            basic_fields=["lead"],
            extra_fields=fields,
            key_field="key",
            template="- **{key}** — {name} ({basics})"
        )

    @mcp.tool()
    async def get_queue(
        ctx: Context,
        queue_key: str,
        expand: str | None = None,
    ) -> str:
        """Get queue details.

        Args:
            queue_key: Queue key (e.g. "DEV")
            expand: Expand: "projects", "components", "versions", "all"
        """
        tracker = ctx.lifespan_context["tracker"]
        queue = await tracker.queues.get(queue_key, expand=expand)
        return _format_queue(queue)

    @mcp.tool()
    async def create_queue(
        ctx: Context,
        key: str,
        name: str,
        lead: str,
        default_type: str,
        default_priority: str,
        issue_types_config: list[dict],
        description: str | None = None,
    ) -> str:
        """Create a new queue.

        Args:
            key: Queue key (e.g. "DEV")
            name: Queue name
            lead: Lead user login
            default_type: Default issue type key
            default_priority: Default priority key
            issue_types_config: Issue types config (list of dicts with issueType and workflow keys)
            description: Queue description
        """
        tracker = ctx.lifespan_context["tracker"]
        kwargs = {}
        if description is not None:
            kwargs["description"] = description

        queue = await tracker.queues.create(
            key, name, lead, default_type, default_priority, issue_types_config, **kwargs
        )
        
        # Invalidate queues cache
        await manager.get("queues", tracker.queues.list, force=True)
        
        return _format_queue(queue)

    @mcp.tool()
    async def delete_queue(
        ctx: Context,
        queue_key: str,
    ) -> str:
        """Delete a queue.

        Args:
            queue_key: Queue key (e.g. "DEV")
        """
        tracker = ctx.lifespan_context["tracker"]
        await tracker.queues.delete(queue_key)
        
        # Invalidate queues cache
        await manager.get("queues", tracker.queues.list, force=True)
        
        return f"Queue {queue_key} deleted."

    @mcp.tool()
    async def restore_queue(
        ctx: Context,
        queue_key: str,
    ) -> str:
        """Restore a previously deleted queue.

        Args:
            queue_key: Queue key (e.g. "DEV")
        """
        tracker = ctx.lifespan_context["tracker"]
        await tracker.queues.restore(queue_key)
        
        # Invalidate queues cache
        await manager.get("queues", tracker.queues.list, force=True)
        
        return f"Queue {queue_key} restored."

    @mcp.tool()
    async def list_queue_versions(
        ctx: Context,
        queue_key: str,
    ) -> str:
        """List versions of a queue.

        Args:
            queue_key: Queue key (e.g. "DEV")
        """
        tracker = ctx.lifespan_context["tracker"]
        
        # Using DirectoryManager for versions might be tricky since they are per-queue
        # but our manager supports scope. However, versions are not in DEFAULT_TTLS.
        # Let's use a generic 'queue_versions' dir type if needed, or just keep it simple.
        # Actually, let's just use the API for now or add it to manager.
        
        versions = await tracker.queues.versions.list(queue_key)

        if not versions:
            return f"No versions in queue {queue_key}."

        lines = [f"Versions in {queue_key}:\n"]
        for v in versions:
            vid = v.get("id", "?")
            name = v.get("name", "")
            start = v.get("startDate", "")
            due = v.get("dueDate", "")
            released = v.get("released", False)
            status = "released" if released else "active"
            lines.append(f"- [{vid}] {name} ({start} — {due}) [{status}]")
        return "\n".join(lines)

    @mcp.tool()
    async def create_queue_version(
        ctx: Context,
        queue_key: str,
        name: str,
        description: str | None = None,
        start_date: str | None = None,
        due_date: str | None = None,
    ) -> str:
        """Create a new version for a queue.

        Args:
            queue_key: Queue key
            name: Version name
            description: Version description
            start_date: Start date (YYYY-MM-DD)
            due_date: Due date (YYYY-MM-DD)
        """
        tracker = ctx.lifespan_context["tracker"]
        kwargs = {}
        if description is not None:
            kwargs["description"] = description
        if start_date is not None:
            kwargs["start_date"] = start_date
        if due_date is not None:
            kwargs["due_date"] = due_date

        version = await tracker.queues.versions.create(queue_key, name, **kwargs)
        vid = version.get("id", "?")
        return f"Version created: [{vid}] {name} in queue {queue_key}"

    @mcp.tool()
    async def delete_queue_tag(
        ctx: Context,
        queue_key: str,
        tag: str,
    ) -> str:
        """Delete a tag from a queue.

        Args:
            queue_key: Queue key
            tag: Tag name
        """
        tracker = ctx.lifespan_context["tracker"]
        await tracker.queues.tags.delete(queue_key, tag)
        
        # Invalidate queue tags cache
        await manager.get(
            "queue_tags", 
            lambda: tracker.queues.tags.list(queue_key),
            scope=queue_key,
            force=True
        )
        
        return f"Tag '{tag}' deleted from queue {queue_key}."

    @mcp.tool()
    async def get_queue_user_permissions(
        ctx: Context,
        queue_key: str,
        user_id: str,
    ) -> str:
        """Get user permissions for a queue.

        Args:
            queue_key: Queue key
            user_id: User ID or login
        """
        tracker = ctx.lifespan_context["tracker"]
        perms = await tracker.queues.permissions.get_user(queue_key, user_id)
        
        lines = [f"Permissions for user {user_id} in {queue_key}:\n"]
        for p in perms:
            lines.append(f"- {p}")
        return "\n".join(lines)

    @mcp.tool()
    async def update_queue_permissions(
        ctx: Context,
        queue_key: str,
        create: dict | None = None,
        write: dict | None = None,
        read: dict | None = None,
        grant: dict | None = None,
    ) -> str:
        """Update queue access permissions.

        Args:
            queue_key: Queue key
            create: Permission payload for 'create' action (e.g. {"users": {"add": ["user1"]}})
            write: Permission payload for 'write' action
            read: Permission payload for 'read' action
            grant: Permission payload for 'grant' action
        """
        tracker = ctx.lifespan_context["tracker"]
        kwargs = {}
        if create: kwargs["create"] = create
        if write: kwargs["write"] = write
        if read: kwargs["read"] = read
        if grant: kwargs["grant"] = grant

        await tracker.queues.permissions.update(queue_key, **kwargs)
        return f"Permissions for queue {queue_key} updated."


def _format_queue(q: dict) -> str:
    key = q.get("key", "?")
    name = q.get("name", "")
    lead = q.get("lead", {})
    if isinstance(lead, dict):
        lead = lead.get("display", lead.get("id", "?"))
    
    lines = [
        f"**{key}**: {name}",
        f"Lead: {lead}",
    ]
    
    desc = q.get("description")
    if desc:
        lines.append(f"\nDescription:\n{desc}")
        
    lines.append(f"\nhttps://tracker.yandex.ru/manager/queues/{key}")
    return "\n".join(lines)



from fastmcp import FastMCP, Context


def register_link_tools(mcp: FastMCP):

    @mcp.tool()
    async def link_issues(
        ctx: Context,
        issue_key: str,
        relationship: str,
        target: str,
    ) -> str:
        """Create a link between two issues.

        Args:
            issue_key: Source issue key (e.g. "DEV-123")
            relationship: Link type - one of: "relates", "depends on", "is dependent by",
                "is subtask for", "is parent task for", "duplicates", "is duplicated by",
                "is epic of", "has epic"
            target: Target issue key (e.g. "DEV-456")
        """
        tracker = ctx.lifespan_context["tracker"]
        link = await tracker.issues.links.create(
            issue_key, relationship, target
        )
        return _format_link(link)

    @mcp.tool()
    async def list_links(
        ctx: Context,
        issue_key: str,
    ) -> str:
        """List all links of an issue.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
        """
        tracker = ctx.lifespan_context["tracker"]
        links = await tracker.issues.links.list(issue_key)

        if not links:
            return f"No links on {issue_key}."

        lines = [f"Links on {issue_key} ({len(links)}):\n"]
        for link in links:
            lines.append(_format_link(link))
        return "\n".join(lines)

    @mcp.tool()
    async def delete_link(
        ctx: Context,
        issue_key: str,
        link_id: str,
    ) -> str:
        """Delete a link from an issue.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
            link_id: Link ID
        """
        tracker = ctx.lifespan_context["tracker"]
        await tracker.issues.links.delete(issue_key, link_id)
        return f"Link {link_id} deleted from {issue_key}."


def _format_link(link: dict) -> str:
    link_id = link.get("id", "?")
    rel_type = link.get("type", {})
    if isinstance(rel_type, dict):
        rel_name = rel_type.get("id", "?")
    else:
        rel_name = str(rel_type)

    direction = link.get("direction", "")

    target = link.get("object", {})
    if isinstance(target, dict):
        target_key = target.get("key", "?")
        target_summary = target.get("display", target.get("summary", ""))
    else:
        target_key = str(target)
        target_summary = ""

    status = ""
    if isinstance(target, dict) and "status" in target:
        s = target["status"]
        status = f" [{s.get('display', s.get('key', ''))}]" if isinstance(s, dict) else f" [{s}]"

    return f"- [{link_id}] {rel_name} {direction} {target_key}{status} {target_summary}"



import os
from contextlib import asynccontextmanager
from collections.abc import AsyncIterator

from fastmcp import FastMCP, Context
from YaTrackerApi import YandexTrackerClient

from ya_tracker_mcp.tools.issues import register_issue_tools
from ya_tracker_mcp.tools.comments import register_comment_tools
from ya_tracker_mcp.tools.links import register_link_tools
from ya_tracker_mcp.tools.transitions import register_transition_tools
from ya_tracker_mcp.tools.queues import register_queue_tools
from ya_tracker_mcp.tools.users import register_user_tools
from ya_tracker_mcp.tools.overviews import register_overview_tools
from ya_tracker_mcp.tools.entities import register_entity_tools
from ya_tracker_mcp.tools.boards import register_board_tools
from ya_tracker_mcp.tools.worklog import register_worklog_tools
from ya_tracker_mcp.tools.checklists import register_checklist_tools
from ya_tracker_mcp.tools.attachments import register_attachment_tools
from ya_tracker_mcp.tools.presets import register_preset_tools
from ya_tracker_mcp.tools.team import register_team_tools
from ya_tracker_mcp.tools.directories import register_directory_tools
from ya_tracker_mcp.tools.automations import register_automation_tools
from ya_tracker_mcp.tools.bulk import register_bulk_tools
from ya_tracker_mcp.tools.imports import register_import_tools
from ya_tracker_mcp.tools.filters import register_filter_tools
from ya_tracker_mcp.tools.dashboards import register_dashboard_tools
from ya_tracker_mcp.tools.external_links import register_external_link_tools
from ya_tracker_mcp.resources.static import register_resources
from ya_tracker_mcp.prompts.prompts import register_prompts


@asynccontextmanager
async def lifespan(mcp: FastMCP) -> AsyncIterator[dict]:
    from ya_tracker_mcp import __version__
    print(f"Starting ya-tracker-mcp v{__version__}")
    
    token = os.environ.get("YA_TRACKER_TOKEN") or os.environ["TRACKER_API_KEY"]
    org_id = os.environ.get("YA_TRACKER_ORG_ID") or os.environ["TRACKER_ORG_ID"]
    async with YandexTrackerClient(oauth_token=token, org_id=org_id) as client:
        yield {"tracker": client}


mcp = FastMCP(
    "Yandex Tracker",
    instructions="MCP server for Yandex Tracker API",
    lifespan=lifespan,
)

register_issue_tools(mcp)
register_comment_tools(mcp)
register_link_tools(mcp)
register_transition_tools(mcp)
register_queue_tools(mcp)
register_user_tools(mcp)
register_overview_tools(mcp)
register_entity_tools(mcp)
register_board_tools(mcp)
register_worklog_tools(mcp)
register_checklist_tools(mcp)
register_attachment_tools(mcp)
register_preset_tools(mcp)
register_team_tools(mcp)
register_directory_tools(mcp)
register_automation_tools(mcp)
register_bulk_tools(mcp)
register_import_tools(mcp)
register_filter_tools(mcp)
register_dashboard_tools(mcp)
register_external_link_tools(mcp)
register_resources(mcp)
register_prompts(mcp)


def main():
    mcp.run()


if __name__ == "__main__":
    main()



from fastmcp import FastMCP, Context


def register_transition_tools(mcp: FastMCP):

    @mcp.tool()
    async def list_transitions(
        ctx: Context,
        issue_key: str,
    ) -> str:
        """List available status transitions for an issue.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
        """
        tracker = ctx.lifespan_context["tracker"]
        transitions = await tracker.issues.transitions.list(issue_key)

        if not transitions:
            return f"No transitions available for {issue_key}."

        lines = [f"Available transitions for {issue_key}:\n"]
        for t in transitions:
            tid = t.get("id", "?")
            display = t.get("display", t.get("to", {}).get("display", "?"))
            to_status = t.get("to", {})
            to_key = to_status.get("key", "") if isinstance(to_status, dict) else ""
            lines.append(f"- [{tid}] {display}" + (f" (-> {to_key})" if to_key else ""))
        return "\n".join(lines)

    @mcp.tool()
    async def transition_issue(
        ctx: Context,
        issue_key: str,
        transition_id: str,
        comment: str | None = None,
    ) -> str:
        """Execute a status transition on an issue.

        Use list_transitions first to get available transition IDs.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
            transition_id: Transition ID (from list_transitions)
            comment: Optional comment for the transition
        """
        tracker = ctx.lifespan_context["tracker"]

        kwargs = {}
        if comment is not None:
            kwargs["comment"] = comment

        result = await tracker.issues.transitions.update(
            issue_key, transition_id, **kwargs
        )
        return f"Transition '{transition_id}' executed on {issue_key}.\n{_format_transition_result(result)}"


def _format_transition_result(result) -> str:
    if isinstance(result, list):
        return f"Transitions remaining: {len(result)}"
    if isinstance(result, dict):
        status = result.get("status", {})
        if isinstance(status, dict):
            return f"New status: {status.get('display', status.get('key', '?'))}"
        return f"Result: {result}"
    return str(result) if result else "Done."



from fastmcp import FastMCP, Context
from ..utils.formatters import format_mcp_item, format_mcp_list


def register_worklog_tools(mcp: FastMCP):

    @mcp.tool()
    async def add_worklog(
        ctx: Context,
        issue_key: str,
        start: str,
        duration: str,
        comment: str | None = None,
        fields: list[str] | None = None,
    ) -> str:
        """Add a worklog entry to an issue.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
            start: Start time (YYYY-MM-DDThh:mm:ss.sss+hhmm)
            duration: Duration in ISO 8601 (e.g. "PT2H30M", "P1D", "PT45M")
            comment: Optional comment
            fields: Optional list of fields to show in response.
        """
        tracker = ctx.lifespan_context["tracker"]

        kwargs = {}
        if comment is not None:
            kwargs["comment"] = comment

        entry = await tracker.worklog.create(issue_key, start, duration, **kwargs)
        return _format_worklog_full(entry, fields)

    @mcp.tool()
    async def list_worklog(
        ctx: Context,
        issue_key: str,
        fields: list[str] | None = None,
    ) -> str:
        """List worklog entries for an issue.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
            fields: Optional list of fields to show.
        """
        tracker = ctx.lifespan_context["tracker"]
        entries = await tracker.worklog.list(issue_key)

        return format_mcp_list(
            entries, f"Worklog for {issue_key}",
            basic_fields=["duration", "createdBy", "start"],
            extra_fields=fields,
            name_field="comment",
            template="- [{key}] {basics}: {name}"
        )

    @mcp.tool()
    async def update_worklog(
        ctx: Context,
        issue_key: str,
        worklog_id: str,
        duration: str,
        comment: str | None = None,
        fields: list[str] | None = None,
    ) -> str:
        """Update a worklog entry.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
            worklog_id: Worklog entry ID
            duration: New duration in ISO 8601 (e.g. "PT3H")
            comment: New comment
            fields: Optional list of fields to show in response.
        """
        tracker = ctx.lifespan_context["tracker"]

        kwargs = {}
        if comment is not None:
            kwargs["comment"] = comment

        entry = await tracker.worklog.update(issue_key, worklog_id, duration, **kwargs)
        return _format_worklog_full(entry, fields)

    @mcp.tool()
    async def delete_worklog(
        ctx: Context,
        issue_key: str,
        worklog_id: str,
    ) -> str:
        """Delete a worklog entry.

        Args:
            issue_key: Issue key (e.g. "DEV-123")
            worklog_id: Worklog entry ID
        """
        tracker = ctx.lifespan_context["tracker"]
        await tracker.worklog.delete(issue_key, worklog_id)
        return f"Worklog {worklog_id} deleted from {issue_key}."

    @mcp.tool()
    async def search_worklog(
        ctx: Context,
        created_by: str | None = None,
        created_at_from: str | None = None,
        created_at_to: str | None = None,
        fields: list[str] | None = None,
    ) -> str:
        """Search worklog entries across all issues.

        Args:
            created_by: Filter by author login
            created_at_from: Start date (YYYY-MM-DD or ISO 8601)
            created_at_to: End date (YYYY-MM-DD or ISO 8601)
            fields: Optional list of additional fields to show.
        """
        tracker = ctx.lifespan_context["tracker"]
        kwargs = {}
        if created_by is not None:
            kwargs["created_by"] = created_by
        if created_at_from is not None:
            kwargs["created_at_from"] = created_at_from
        if created_at_to is not None:
            kwargs["created_at_to"] = created_at_to

        entries = await tracker.worklog.search(**kwargs)

        return format_mcp_list(
            entries, "Worklog entries",
            basic_fields=["issue", "duration", "createdBy", "start"],
            extra_fields=fields,
            name_field="comment",
            template="- [{key}] {basics}: {name}"
        )


def _format_worklog_full(entry: dict, extra_fields: list[str] | None = None) -> str:
    return format_mcp_item(
        entry, "Worklog",
        basic_fields=["duration", "createdBy", "start"],
        extra_fields=extra_fields,
        name_field="comment",
        template="[{key}] {basics}: {name}",
        description_field=None
    )



import os
import time
import yaml
from datetime import datetime

# Path to the directories configuration and data
CONFIG_PATH = os.path.join(os.path.dirname(__file__), "..", "config", "directories.yaml")

DEFAULT_TTLS = {
    "issue_types": 86400,    # 24h
    "statuses": 86400,       # 24h
    "priorities": 86400,     # 24h
    "resolutions": 86400,    # 24h
    "global_fields": 86400,  # 24h
    "field_categories": 86400, # 24h
    "queues": 3600,          # 1h
    "components": 3600,      # 1h
    "boards": 3600,          # 1h
    "sprints": 1800,         # 30m
    "users": 3600,           # 1h
    "queue_fields": 1800,    # 30m
    "queue_tags": 900,       # 15m
}

class DirectoryManager:
    def __init__(self):
        self._load()

    def _load(self):
        if not os.path.exists(CONFIG_PATH):
            self.data = {}
            self.config = {"ttls": DEFAULT_TTLS.copy()}
            return

        try:
            with open(CONFIG_PATH, "r") as f:
                full_data = yaml.safe_load(f) or {}
                self.data = full_data.get("data", {})
                self.config = full_data.get("config", {"ttls": DEFAULT_TTLS.copy()})
                if "ttls" not in self.config:
                    self.config["ttls"] = DEFAULT_TTLS.copy()
        except Exception:
            self.data = {}
            self.config = {"ttls": DEFAULT_TTLS.copy()}

    def _save(self):
        os.makedirs(os.path.dirname(CONFIG_PATH), exist_ok=True)
        try:
            with open(CONFIG_PATH, "w") as f:
                yaml.dump({
                    "config": self.config,
                    "data": self.data,
                    "meta": {
                        "last_saved": datetime.now().isoformat()
                    }
                }, f, allow_unicode=True, sort_keys=False)
        except Exception as e:
            print(f"Error saving directory cache: {e}")

    def get_ttl(self, directory: str) -> int:
        return self.config["ttls"].get(directory, DEFAULT_TTLS.get(directory, 3600))

    def set_ttl(self, directory: str, ttl: int):
        self.config["ttls"][directory] = ttl
        self._save()

    def _get_cache_key(self, directory: str, scope: str | None = None) -> str:
        return f"{directory}:{scope}" if scope else directory

    async def get(self, directory: str, fetch_func, scope: str | None = None, force: bool = False):
        key = self._get_cache_key(directory, scope)
        entry = self.data.get(key)
        now = time.time()
        ttl = self.get_ttl(directory)

        if not force and entry and (now - entry.get("timestamp", 0) < ttl):
            return entry.get("values")

        # Fetch from API
        values = await fetch_func()
        
        # Save to cache
        self.data[key] = {
            "values": values,
            "timestamp": now,
            "updated_at": datetime.now().isoformat()
        }
        self._save()
        return values

    def get_status(self):
        status = []
        now = time.time()
        for key, entry in self.data.items():
            ts = entry.get("timestamp", 0)
            dir_type = key.split(":")[0]
            ttl = self.get_ttl(dir_type)
            expires_in = max(0, int(ts + ttl - now))
            
            status.append({
                "key": key,
                "count": len(entry.get("values", [])),
                "updated_at": entry.get("updated_at"),
                "ttl": ttl,
                "expires_in_sec": expires_in,
                "is_expired": expires_in == 0
            })
        return status

    async def sync_all(self, registry: dict):
        """
        Sync all directories using a registry of fetch functions.
        registry: { "directory_name": fetch_func, "directory_name:scope": fetch_func }
        """
        results = []
        for key, fetch_func in registry.items():
            parts = key.split(":", 1)
            directory = parts[0]
            scope = parts[1] if len(parts) > 1 else None
            try:
                await self.get(directory, fetch_func, scope=scope, force=True)
                results.append(f"✅ {key} synced")
            except Exception as e:
                results.append(f"❌ {key} failed: {str(e)}")
        return results

# Singleton instance
manager = DirectoryManager()
